// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <atomic>	 // for atomic

#include "PrivateSemaphore.hpp"	 // for PrivateSemaphore

class Context;

class BinaryPrivateSemaphore : public PrivateSemaphore {
 private:
	enum State {
		initial,
		signaled,
		blocked,
	};

	std::atomic<State> bpsState;

	// It would be OK to not initialize this member, since it is only
	// set after the State is set accordingly and then only read if
	// this state is found. However clang-tidy then starts to complain
	// about clang-analyzer-optin.cplusplus.UninitializedObject, and
	// furthermore, all call sites of wait() yield a
	// clang-analyzer-optin.cplusplus.VirtualCall (it appears
	// clang-tidy only checks if the object is fully initialized, not
	// if the vcall is actually performed in a constructor.
	Context* blockedContext = nullptr;

 protected:
	auto signalInternal() -> Context* override;

 public:
	BinaryPrivateSemaphore() : bpsState(initial) {}

	void wait() override;

	void reset() { bpsState = initial; };
};

using BPS = BinaryPrivateSemaphore;
