// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include "Common.hpp"
#include "emper-common.h"

class Runtime;

class Worker {
 private:
	static thread_local Worker* currentWorker;

	ALIGN_TO_CACHE_LINE unsigned int seed;

	const workerid_t workerId;

	Worker(workerid_t workerId, unsigned int seed) : seed(seed), workerId(workerId) {}

	void setWorker();

	friend Runtime;

 public:
	[[nodiscard]] auto getWorkerId() const -> workerid_t { return workerId; }

	// https://stackoverflow.com/a/3747462/194894
	inline auto nextRand() -> int {
		seed = 214013 * seed + 2531011;
		auto shifted_seed = seed >> 16;
		return *reinterpret_cast<int*>(&shifted_seed) & 0x7FFF;
	}

	static inline auto isWorkerThread() -> bool { return currentWorker != nullptr; }

	static inline auto getCurrentWorker() -> Worker* { return currentWorker; }

	// https://stackoverflow.com/a/3747462/194894
	static inline auto rand() -> int {
		Worker* currentWorker = getCurrentWorker();
		return currentWorker->nextRand();
	}

	static inline auto getCurrentWorkerId() -> workerid_t {
		Worker* currentWorker = getCurrentWorker();
		return currentWorker->getWorkerId();
	}
};
