// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Fischer
#pragma once

#include <atomic>

#include "Common.hpp"

namespace emper::lib::adt {

class ALIGN_TO_CACHE_LINE AtomicTryLock {
 private:
	std::atomic<bool> locked;

 public:
	AtomicTryLock(bool locked = false) : locked(locked) {}

	auto try_lock() -> bool {
		bool previously_locked = locked.exchange(true, std::memory_order_acquire);
		return !previously_locked;
	}

	void unlock() { locked.store(false, std::memory_order_release); }
};
}	 // namespace emper::lib::adt
