// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Fischer
#pragma once

#include <iterator>
#include <mutex>
#include <set>

namespace emper::lib::adt {

template <class Key, class Compare = std::less<Key>, class Allocator = std::allocator<Key>>
class LockedSet {
 private:
	std::mutex _mutex;
	std::set<Key, Compare, Allocator> _set;

 public:
	auto insert(const Key& item)
			-> std::pair<typename std::set<Key, Compare, Allocator>::iterator, bool> {
		std::lock_guard<std::mutex> lock(_mutex);
		return _set.insert(item);
	}

	template <class InputIt>
	void insert(InputIt first, InputIt last) {
		std::lock_guard<std::mutex> lock(_mutex);
		_set.insert(first, last);
	}

	auto erase(const Key& key) -> size_t {
		std::lock_guard<std::mutex> lock(_mutex);
		return _set.erase(key);
	}
};
}	 // namespace emper::lib::adt
