// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus
#pragma once

#include <cstddef>	// for size_t
#include <cstdint>
#include <utility>

#include "NextFiberResult.hpp"
#include "Scheduler.hpp"
#include "emper-common.h"

#ifdef EMPER_LOCKED_WS_QUEUE
#include "lib/adt/LockedQueue.hpp"
#else
#include "lib/adt/WsClQueue.hpp"
#endif

class Fiber;
class Runtime;
class AbstractWorkStealingStrategy;

class AbstractWorkStealingScheduler : public Scheduler {
	template <size_t SIZE>
#ifdef EMPER_LOCKED_WS_QUEUE
	using WsQueue = adt::LockedQueue<Fiber*, SIZE>;
#else
	using WsQueue = adt::WsClQueue<Fiber*, SIZE>;
#endif

 private:
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wattributes"
	AbstractWorkStealingStrategy& abstractWorkStealingStrategy
#ifndef EMPER_STATS
			ATTR_UNUSED
#endif
			;
#pragma GCC diagnostic pop

 public:
	static const int QUEUE_SIZE = 1024;

	enum class FiberSource : uintptr_t {
		local,
		stolen,
		anywhereQueue,
	};

 protected:
	WsQueue<QUEUE_SIZE>** queues;
	static thread_local WsQueue<QUEUE_SIZE> queue;

	void scheduleViaWorkStealing(Fiber& fiber);

	auto nextFiberViaWorkStealing() -> std::pair<Fiber*, FiberSource>;

	auto nextFiberResultViaWorkStealing() -> NextFiberResult;

 public:
	AbstractWorkStealingScheduler(Runtime& runtime,
																AbstractWorkStealingStrategy& abstractWorkStealingStrategy);
};
