// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#pragma once

#include <atomic>
#include <cstdint>
#include <memory>

#include "strategies/AbstractWorkStealingScheduler.hpp"
#include "strategies/AbstractWorkStealingStrategy.hpp"
#include "strategies/laws/LawsDispatcher.hpp"
#include "strategies/laws/LawsScheduler.hpp"

class LawsStrategyFactory;
class LawsStrategyStats;
class Runtime;
class RuntimeStrategyStats;

class LawsStrategy : public AbstractWorkStealingStrategy {
 private:
	enum class FiberSource : uintptr_t {
		local = static_cast<uintptr_t>(AbstractWorkStealingScheduler::FiberSource::local),
		stolen = static_cast<uintptr_t>(AbstractWorkStealingScheduler::FiberSource::stolen),
		anywhereQueue =
				static_cast<uintptr_t>(AbstractWorkStealingScheduler::FiberSource::anywhereQueue),
		fromPriority,
	};

	LawsScheduler scheduler;
	LawsDispatcher dispatcher;

	// TODO: Align those all to cache line!
	std::atomic<std::uint64_t> scheduledFibersToRemotePriority;
	std::atomic<std::uint64_t> dispatchedFiberFromPriority;
	std::atomic<std::uint64_t> dispatchedFiberFromLocal;
	std::atomic<std::uint64_t> dispatchedFiberStolen;
	std::atomic<std::uint64_t> dispatchedFiberFromAnywhere;

	LawsStrategy(Runtime& runtime);

	auto getScheduler() -> LawsScheduler& override;

	auto getDispatcher() -> LawsDispatcher& override;

 public:
	auto getStats() -> std::shared_ptr<RuntimeStrategyStats> override;

	friend LawsScheduler;
	friend LawsDispatcher;
	friend LawsStrategyStats;
	friend LawsStrategyFactory;
};
